/*
 * Created on 20.2.2005
 *
 */
package net.sf.tomp.el.author;

import java.io.File;

import net.sf.tomp.el.author.description.Described;
import net.sf.tomp.el.author.description.Description;

/**
 * @author <a href="mailto:tomp@fi.muni.cz">RNDr. Tomáš Pitner, Ph.D.</a>
 *
 *
 */
public class Course implements Described {
    
    public static final String SRC_DIR = "src";
    
    public static final String MAIN_FILE = "main.xml";
    
    public static final String MAIN_INFO_FILE = "main-info.xml";
    
    private CourseTree courseTree;
    
    private String id;
    
    private Lectures lectures;
    private Tasks tasks;
    private Practices practices;
    private Projects projects;
    
    private Description description;
    
    private String nameInTree;
    
    /**
     * Creates a new instance of Course, sets its CourseTree and its ID.
     * Sets the default Description(title, body). Created a new instance
     * of its Lectures.
     * @param t CoursTree of the new Course
     * @param id Course's ID.
     */
    public Course(CourseTree t, String id) {
        this.courseTree = t;
        this.id = id;
        lectures = new Lectures(this);
        tasks = new Tasks(this);
        practices = new Practices(this);
        projects = new Projects(this);
        description = new Description("title of a course", "body of a course");
        nameInTree = getId() + " (" + getDescription().getTitle() + ")";
    }
    
    /**
     * Returns the name in the GUI tree.
     * @return name in the GUI tree
     */
    public String getNameInTree() {
        return nameInTree;
    }
    
    /**
     * Sets the name in the GUI tree.
     * @param n name in the GUI tree
     */
    public void setNameInTree(String n) {
        nameInTree = n;
    }
    
    /**
     * Updates the name in the GUI tree after objects's description change
     * @return update name in the GUI tree
     */
    public String updateNameInTree() {
        nameInTree = getId() + " (" + getDescription().getTitle() + ")";
        return nameInTree;
    }
    
    /**
     * Returns the CourseTree directory.
     * @return CourseTree directory
     */
    public File getDirectory() {
        return new File(courseTree.getDirectory(), id);
    }
    
    /**
     * Returns the source directory.
     * @return Source directory
     */
    public File getSrcDirectory() {
        return new File(getDirectory(), SRC_DIR);
    }
    
    /**
     * Returns the main XML document.
     * @return Main XML file.
     */
    public File getFile() {
        return new File(getSrcDirectory(), MAIN_FILE);
    }
    
    /**
     * Returns the main informative XML document.
     * @return Main informative XML file.
     */
    public File getInfoFile() {
        return new File(getSrcDirectory(), MAIN_INFO_FILE);
    }
    
    /**
     * Returns the course's ID.
     * @return course's ID.
     */
    public String getId() {
        return id;
    }
    
    /**
     * Sets course's ID.
     * @param id new id to set
     */
    public void setId(String id) {
        this.id = id;
    }
    
    /**
     * Returns the courseTree.
     * @return CourseTree
     */
    public CourseTree getCourseTree() {
        return courseTree;
    }
    
    /**
     * Returns the lectures.
     * @return Lectures
     */
    public Lectures getLectures() {
        return lectures;
    }
    
    /**
     * Returns the tasks.
     * @return Tasks
     */
    public Tasks getTasks() {
        return tasks;
    }
    
    /**
     * Returns the practices.
     * @return Lectures
     */
    public Practices getPractices() {
        return practices;
    }
    
    /**
     * Returns the projects.
     * @return Lectures
     */
    public Projects getProjects() {
        return projects;
    }
    
    /**
     * Creates string of course's ID and its lectures.
     * @return String containing course's ID and its lectures.
     */
    public String toString() {
        return "course " + id + "\n" + getLectures();
    }
    /**
     * Returns the description.
     * @return Description
     */
    public Description getDescription() {
        return description;
    }
    /**
     * Sets the description set.
     * @param description The description to set.
     */
    public void setDescription(Description description) {
        this.description = description;
    }
}
