/*
 * Created on 24.2.2005
 *
 */
package net.sf.tomp.el.author;

import java.io.File;

import net.sf.tomp.el.author.description.Described;
import net.sf.tomp.el.author.description.Description;

/**
 * @author <a href="mailto:tomp@fi.muni.cz">RNDr. Tomáš Pitner, Ph.D.</a>
 *
 *
 */
public class Lecture implements Described {
    
    private Lectures lectures;
    
    private String id;
    
    private Description description;
    
    private String nameInTree;
    
    /**
     * Returns the Lecture's XML file.
     * @return Lecture's file
     */
    public File getFile() {
        return new File(lectures.getDirectory(), id + ".xml");
    }
    
    /**
     * Returns lectures.
     * @return Lectures.
     */
    public Lectures getLectures() {
        return lectures;
    }
    
    
    /**
     * Returns the name in the GUI tree.
     * @return name in the GUI tree
     */
    public String getNameInTree() {
        return nameInTree;
    }
    
    /**
     * Sets the name in the GUI tree.
     * @param n name in the GUI tree
     */
    public void setNameInTree(String n) {
        nameInTree = n;
    }
    
    /**
     * Updates the name in the GUI tree after objects's description change
     * @return update name in the GUI tree
     */
    public String updateNameInTree() {
        nameInTree = getId() + " (" + getDescription().getTitle() + ")";
        return nameInTree;
    }
    
    /**
     * Creates a new instance of Lecture and sets its ID.
     * Initialize its lectures and creates default description.
     * @param l Lectures
     * @param id Lecture's ID
     */
    public Lecture(Lectures l, String id) {
        lectures = l;
        this.id = id;
        description = new Description("title of a lecture", "body of a lecture");
        nameInTree = getId() + " (" + getDescription().getTitle() + ")";
    }
    
    /**
     * Returns lecture's ID.
     * @return ID
     */
    public String getId() {
        return id;
    }
    
    /**
     * Sets lecture's ID.
     * @param id new id to set
     */
    public void setId(String id) {
        this.id = id;
    }    
    
    /**
     * Creates string of lecture.
     * @return String containing "lecture &lt;ID&gt;"
     */
    public String toString() {
        return "lecture " + id;
    }
    
    /**
     * Returns the description.
     * @return Lecture's description
     */
    public Description getDescription() {
        return description;
    }
    
    /**
     * Sets the lecture's description
     * @param description The description to set.
     */
    public void setDescription(Description description) {
        this.description = description;
    }
}
