/*
 * Created on 24.2.2005
 *
 */
package net.sf.tomp.el.author;

import java.io.File;
import java.io.IOException;
import java.util.Collection;
import java.util.TreeMap;
import java.util.Iterator;
import java.util.SortedMap;

import net.sf.tomp.el.author.description.Described;
import net.sf.tomp.el.author.description.Description;

/**
 * @author <a href="mailto:tomp@fi.muni.cz">RNDr. Tomáš Pitner, Ph.D.</a>
 *
 *
 */
public class Lectures implements Described {

    private Description description;

    private Course course;

    private SortedMap<String, Lecture> lectures;

    /**
     * Creates a new instance of Lectures and sets its course.
     * Creates a new instance of its TreeMap lectures.
     * @param c Course containing these lectures.
     */
    public Lectures(Course c) {
        this.course = c;
        lectures = new TreeMap<String, Lecture>();
        description = new Description("title of a lectures",
                "body of a lectures");
    }

    /**
     * Returns Lectures' directory.
     * @return Lectures' directory
     */
    public File getDirectory() {
        return new File(course.getSrcDirectory(), "lectures");
    }

    /**
     * Returns the lectures' XML file.
     * @return Lectures' file
     */
    public File getFile() {
        return new File(course.getSrcDirectory(), "lectures.xml");
    }

    /**
     * Returns the lectures' info XML file.
     * @return Lectures' info file
     */
    public File getInfoFile() {
        return new File(course.getSrcDirectory(), "lectures-info.xml");
    }

    /**
     * Returns lectures' values.
     * @return Collection of lectures' values
     */
    public Collection<Lecture> getLectures() {
        return lectures.values();
    }

    /**
     * Adds a lecture to lectures.
     * @param c Lecture to add
     */
    public void addLecture(Lecture c) {
        lectures.put(c.getId(), c);
    }

    /**
     * Creates a new lecture and puts it into lectures.
     * @param id ID of new lecture
     * @return Created lecture
     */
    public Lecture newLecture(String id) throws IOException {
        Lecture c = new Lecture(this, id);
        c.getFile().createNewFile();
        lectures.put(c.getId(), c);
        return c;
    }

    /**
     * Remove the lecture from lectures collection and deletes its XML file.
     * @param l lecture to remove
     */
    public void removeLecture(Lecture l) {
        l.getFile().delete();
        lectures.remove(l.getId());
    }

    /**
     * Returns lecture with selected ID.
     * @param id ID of lecture to return
     * @return Lecture with selected ID
     */
    public Lecture getLecture(String id) {
        if (id == null) {
            return null;
        }
        return lectures.get(id);
    }

    /**
     * Returns the description.
     * @return Lecture's description
     */
    public Description getDescription() {
        return description;
    }

    /**
     * Sets the lecture's description
     * @param description The description to set.
     */
    public void setDescription(Description description) {
        this.description = description;
    }

    /**
     * Returns first lecture in lectures map.
     * @return First lecture in lectures.
     */
    public Lecture getFirstLecture() {
        if (!lectures.isEmpty()) {
            return lectures.get(lectures.firstKey());
        } else {
            return null;
        }
    }

    /** Creates string of lectures' IDs.
     * @return String containing lectures' IDs (comma-separated)
     */
    public String toString() {
        StringBuffer sb = new StringBuffer();
        for (Iterator<Lecture> i = getLectures().iterator(); i.hasNext();) {
            sb.append(i.next());
            if (i.hasNext()) {
                sb.append(", ");
            }
        }
        return "lectures [" + sb + "]\n";
    }
}
