/*
 * Created on 24.2.2005
 *
 */
package net.sf.tomp.el.author;

import java.io.File;

import net.sf.tomp.el.author.description.Described;
import net.sf.tomp.el.author.description.Description;

/**
 * @author <a href="mailto:xkyselak@fi.muni.cz">Martin Kyselák</a>
 *
 *
 */
public class Practice implements Described {
    
    private Practices practices;
    
    private String id;
    
    private Description description;
    
    private String nameInTree;
    
    /**
     * Returns the Practice's XML file.
     * @return Practice's file
     */
    public File getFile() {
        return new File(practices.getDirectory(), id + ".xml");
    }
    
    /**
     * Returns the name in the GUI tree.
     * @return name in the GUI tree
     */
    public String getNameInTree() {
        return nameInTree;
    }
    
    /**
     * Sets the name in the GUI tree.
     * @param n name in the GUI tree
     */
    public void setNameInTree(String n) {
        nameInTree = n;
    }

    /**
     * Updates the name in the GUI tree after objects's description change
     * @return update name in the GUI tree
     */
    public String updateNameInTree() {
        nameInTree = getId() + " (" + getDescription().getTitle() + ")";
        return nameInTree;
    }    
    
    /**
     * Returns practices.
     * @return Practices.
     */
    public Practices getPractices() {
        return practices;
    }
    
    /**
     * Creates a new instance of Practice and sets its ID.
     * Initialize its practices and creates default description.
     * @param p Practices
     * @param id Practice's ID
     */
    public Practice(Practices p, String id) {
        practices = p;
        this.id = id;
        description = new Description("title of a practice",
                "body of a practice");
        nameInTree = getId() + " (" + getDescription().getTitle() + ")";
    }
    
    /**
     * Returns practice's ID.
     * @return ID
     */
    public String getId() {
        return id;
    }
    
    /**
     * Sets practice's ID.
     * @param id new id to set
     */
    public void setId(String id) {
        this.id = id;
    }      
    
    /**
     * Creates string of practice.
     * @return String containing "practice &lt;ID&gt;"
     */
    public String toString() {
        return "practice " + id;
    }
    
    /**
     * Returns the description.
     * @return Practice's description
     */
    public Description getDescription() {
        return description;
    }
    
    /**
     * Sets the practice's description
     * @param description The description to set.
     */
    public void setDescription(Description description) {
        this.description = description;
    }
}
