/*
 * Created on 24.2.2005
 *
 */
package net.sf.tomp.el.author;

import java.io.File;
import java.io.IOException;
import java.util.Collection;
import java.util.TreeMap;
import java.util.Iterator;
import java.util.SortedMap;

import net.sf.tomp.el.author.description.Described;
import net.sf.tomp.el.author.description.Description;

/**
 * @author <a href="mailto:xkyselak@fi.muni.cz">Martin Kyselák</a>
 *
 *
 */
public class Practices implements Described {

    private Description description;

    private Course course;

    private SortedMap<String, Practice> practices;

    /**
     * Creates a new instance of Practices and sets its course.
     * Creates a new instance of its TreeMap practices.
     * @param c Course containing these practices.
     */
    public Practices(Course c) {
        this.course = c;
        practices = new TreeMap<String, Practice>();
        description = new Description("title of a practices",
                "body of a practices");
    }

    /**
     * Returns Practices' directory.
     * @return Practices' directory
     */
    public File getDirectory() {
        return new File(course.getSrcDirectory(), "practices");
    }

    /**
     * Returns the practices' XML file.
     * @return Practices' file
     */
    public File getFile() {
        return new File(course.getSrcDirectory(), "practices.xml");
    }

    /**
     * Returns the practices' info XML file.
     * @return Practices' info file
     */
    public File getInfoFile() {
        return new File(course.getSrcDirectory(), "practices-info.xml");
    }

    /**
     * Returns practices' values.
     * @return Collection of practices' values
     */
    public Collection<Practice> getPractices() {
        return practices.values();
    }

    /**
     * Adds a practice to practices.
     * @param c Practice to add
     */
    public void addPractice(Practice c) {
        practices.put(c.getId(), c);
    }

    /**
     * Creates a new practice and puts it into practices.
     * @param id ID of new practice
     * @return Created practice
     */
    public Practice newPractice(String id) throws IOException {
        Practice p = new Practice(this, id);
        p.getFile().createNewFile();
        practices.put(p.getId(), p);
        return p;
    }

    /**
     * Returns the description.
     * @return Lecture's description
     */
    public Description getDescription() {
        return description;
    }

    /**
     * Sets the lecture's description
     * @param description The description to set.
     */
    public void setDescription(Description description) {
        this.description = description;
    }

    /**
     * Remove the practice from practices collection and deletes its XML file.
     * @param l practice to remove
     */
    public void removePractice(Practice l) {
        l.getFile().delete();
        practices.remove(l.getId());
    }

    /**
     * Returns practice with selected ID.
     * @param id ID of practice to return
     * @return Practice with selected ID
     */
    public Practice getPractice(String id) {
        if (id == null)
            return null;
        return practices.get(id);
    }

    /**
     * Returns first practice in practices map.
     * @return First practice in practices.
     */
    public Practice getFirstPractice() {
        if (!practices.isEmpty()) {
            return practices.get(practices.firstKey());
        } else {
            return null;
        }
    }

    /** Creates string of practices' IDs.
     * @return String containing practices' IDs (comma-separated)
     */
    public String toString() {
        StringBuffer sb = new StringBuffer();
        for (Iterator<Practice> i = getPractices().iterator(); i.hasNext();) {
            sb.append(i.next());
            if (i.hasNext()) {
                sb.append(", ");
            }
        }
        return "practices [" + sb + "]\n";
    }
}
