/*
 * Created on 24.2.2005
 *
 */
package net.sf.tomp.el.author;

import java.io.File;

import net.sf.tomp.el.author.description.Described;
import net.sf.tomp.el.author.description.Description;

/**
 * @author <a href="mailto:xkyselak@fi.muni.cz">Martin Kyselák</a>
 *
 *
 */
public class Project implements Described {
    
    private Projects projects;
    
    private String id;
    
    private Description description;
    
    private String nameInTree;
    
    /**
     * Returns the Project's XML file.
     * @return Project's file
     */
    public File getFile() {
        return new File(projects.getDirectory(), id + ".xml");
    }
    
    /**
     * Returns the name in the GUI tree.
     * @return name in the GUI tree
     */
    public String getNameInTree() {
        return nameInTree;
    }
    
    /**
     * Sets the name in the GUI tree.
     * @param n name in the GUI tree
     */
    public void setNameInTree(String n) {
        nameInTree = n;
    }
    
    /**
     * Updates the name in the GUI tree after objects's description change
     * @return update name in the GUI tree
     */
    public String updateNameInTree() {
        nameInTree = getId() + " (" + getDescription().getTitle() + ")";
        return nameInTree;
    }    
    
    /**
     * Returns project.
     * @return Projects.
     */
    public Projects getProjects() {
        return projects;
    }
    
    /**
     * Creates a new instance of Project and sets its ID.
     * Initialize its projects and creates default description.
     * @param l Projects
     * @param id Project's ID
     */
    public Project(Projects l, String id) {
        projects = l;
        this.id = id;
        description = new Description("title of a project", "body of a project");
        nameInTree = getId() + " (" + getDescription().getTitle() + ")";
    }
    
    /**
     * Returns project's ID.
     * @return ID
     */
    public String getId() {
        return id;
    }
    
    /**
     * Sets project's ID.
     * @param id new id to set
     */
    public void setId(String id) {
        this.id = id;
    }      
    
    /**
     * Creates string of project.
     * @return String containing "project &lt;ID&gt;"
     */
    public String toString() {
        return "project " + id;
    }
    
    /**
     * Returns the description.
     * @return Project's description
     */
    public Description getDescription() {
        return description;
    }
    
    /**
     * Sets the project's description
     * @param description The description to set.
     */
    public void setDescription(Description description) {
        this.description = description;
    }
}
