/*
 * Created on 24.2.2005
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package net.sf.tomp.el.author;

import java.io.File;
import java.io.IOException;
import java.util.Collection;
import java.util.TreeMap;
import java.util.Iterator;
import java.util.SortedMap;

import net.sf.tomp.el.author.description.Described;
import net.sf.tomp.el.author.description.Description;
/**
 * @author <a href="mailto:xkyselak@fi.muni.cz">Martin Kyselák</a>
 *
 *
 */
public class Projects implements Described {
    
    private Course course;
    
    private SortedMap<String, Project> projects;
    
    private Description description;
    
    /**
     * Creates a new instance of Projects and sets its course.
     * Creates a new instance of its TreeMap Projects.
     * @param c Course containing these projects.
     */
    public Projects(Course c) {
        this.course = c;
        projects = new TreeMap<String, Project>();
        description = new Description(
                "title of a projects", "body of a projects");
    }
    
    /**
     * Returns Projects' directory.
     * @return Projects' directory
     */
    public File getDirectory() {
        return new File(course.getSrcDirectory(), "projects");
    }
    
    /**
     * Returns the projects' XML file.
     * @return Projects' file
     */
    public File getFile() {
        return new File(course.getSrcDirectory(), "projects.xml");
    }
    
    /**
     * Returns the projects' info XML file.
     * @return Projects' info file
     */
    public File getInfoFile() {
        return new File(course.getSrcDirectory(), "projects-info.xml");
    }
    
    /**
     * Returns projects' values.
     * @return Collection of projects' values
     */
    public Collection<Project> getProjects() {
        return projects.values();
    }
    
    /**
     * Adds a project to projects.
     * @param c Project to add
     */
    public void addProject(Project c) {
        projects.put(c.getId(), c);
    }
    
    /**
     * Creates a new project and puts it into lprojects.
     * @param id ID of new project
     * @return Created project J
     */
    public Project newProject(String id) throws IOException {
        Project p = new Project(this, id);
        p.getFile().createNewFile();
        projects.put(p.getId(), p);
        return p;
    }
    
    /**
     * Remove the project from projects collection and deletes its XML file.
     * @param p project to remove
     */
    public void removeProject(Project p) {
        p.getFile().delete();
        projects.remove(p.getId());
    }
    
    /**
     * Returns project with selected ID.
     * @param id ID of project to return
     * @return Project with selected ID
     */
    public Project getProject(String id) {
        if (id == null)
            return null;
        return projects.get(id);
    }
    
    /**
     * Returns first project in projects map.
     * @return First project in projects.
     */
    public Project getFirstProject() {
        if (!projects.isEmpty()) {
            return projects.get(projects.firstKey());
        } else {
            return null;
        }
    }
    
    /**
     * Returns the description.
     * @return Lecture's description
     */
    public Description getDescription() {
        return description;
    }
    
    /**
     * Sets the lecture's description
     * @param description The description to set.
     */
    public void setDescription(Description description) {
        this.description = description;
    }
    
    /** Creates string of proejcts' IDs.
     * @return String containing projects' IDs (comma-separated)
     */
    public String toString() {
        StringBuffer sb = new StringBuffer();
        for (Iterator<Project> i = getProjects().iterator(); i.hasNext();) {
            sb.append(i.next());
            if(i.hasNext()) {
                sb.append(", ");
            }
        }
        return "projects ["+sb+"]\n";
    }
}
