/*
 * Created on 24.2.2005
 *
 */
package net.sf.tomp.el.author;

import java.io.File;

import net.sf.tomp.el.author.description.Described;
import net.sf.tomp.el.author.description.Description;

/**
 * @author <a href="mailto:tomp@fi.muni.cz">RNDr. Tomáš Pitner, Ph.D.</a>
 *
 *
 */
public class Task implements Described {
    
    private Tasks tasks;
    
    private String id;
    
    private Description description;
    
    private String nameInTree;
    
    /**
     * Returns the task's XML file.
     * @return Task's file
     */
    public File getFile() {
        return new File(tasks.getDirectory(), id + ".xml");
    }
    
    /**
     * Returns the name in the GUI tree.
     * @return name in the GUI tree
     */
    public String getNameInTree() {
        return nameInTree;
    }
    
    /**
     * Sets the name in the GUI tree.
     * @param n name in the GUI tree
     */
    public void setNameInTree(String n) {
        nameInTree = n;
    }
    
    /**
     * Updates the name in the GUI tree after objects's description change
     * @return update name in the GUI tree
     */
    public String updateNameInTree() {
        nameInTree = getId() + " (" + getDescription().getTitle() + ")";
        return nameInTree;
    }
    
    /**
     * Returns the tasks.
     * @return Tasks
     */
    public Tasks getTasks() {
        return tasks;
    }
    
    /**
     * Creates a new instance  of Task and sets its ID and Tasks.
     *
     * @param l Tasks of the new instance
     * @param id New task ID
     */
    public Task(Tasks l, String id) {
        tasks = l;
        this.id = id;
        description = new Description("title of a task", "body of a task");
        nameInTree = getId() + " (" + getDescription().getTitle() + ")";
    }
    
    /**
     * Returns task's ID
     * @return Task's ID
     */
    public String getId() {
        return id;
    }
    
    /**
     * Sets task's ID.
     * @param id new id to set
     */
    public void setId(String id) {
        this.id = id;
    }
    
    /**
     * Creates string of task.
     * @return String containing "task &lt;ID&gt;"
     */
    public String toString() {
        return "task "+id;
    }
    /**
     * Returns the task's description.
     * @return Task's description
     */
    public Description getDescription() {
        return description;
    }
    /**
     * Sets the task's description.
     * @param description The description to set.
     */
    public void setDescription(Description description) {
        this.description = description;
    }
}
