/*
 * Created on 24.2.2005
 *
 */
package net.sf.tomp.el.author;

import java.io.File;
import java.io.IOException;
import java.util.Collection;
import java.util.SortedMap;
import java.util.Iterator;
import java.util.TreeMap;

import net.sf.tomp.el.author.description.Described;
import net.sf.tomp.el.author.description.Description;

/**
 * @author <a href="mailto:tomp@fi.muni.cz">RNDr. Tomáš Pitner, Ph.D.</a>
 *
 *
 */
public class Tasks implements Described {
    
    private Description description;
    
    private Course course;
    
    private SortedMap<String, Task> tasks;
    
    /**
     * Creates a new instance of Tasks, sets it course.
     * Creates a new instance of its TreeMap tasks.
     * @param c Course containing these tasks.
     */
    public Tasks(Course c) {
        this.course = c;
        tasks = new TreeMap<String, Task>();
        description = new Description("title of a tasks", "body of a tasks");        
    }
    
    /**
     * Returns tasks' directory.
     * @return Tasks' directory
     */
    public File getDirectory() {
        return new File(course.getSrcDirectory(), "tasks");
    }
    
    /**
     * Returns the tasks' XML file.
     * @return Tasks's file
     */
    public File getFile() {
        return new File(course.getSrcDirectory(), "tasks.xml");
    }
    
    /**
     * Returns the tasks' info XML file.
     * @return Tasks' info file
     */
    public File getInfoFile() {
        return new File(course.getSrcDirectory(), "tasks-info.xml");
    }
    
    /**
     * Returns tasks' values.
     * @return Collection of tasks' values.
     */
    public Collection<Task> getTasks() {
        return tasks.values();
    }
    
    /**
     * Adds a task to tasks.
     * @param c Task to add
     */
    public void addTask(Task c) {
        tasks.put(c.getId(), c);
    }
    
    /**
     * Returns task with selected ID.
     * @param id ID of task to return
     * @return Task with selected ID
     */
    public Task getTask(String id) {
        if (id == null)
            return null;
        return tasks.get(id);
    }
    
    /**
     * Returns the description.
     * @return Lecture's description
     */
    public Description getDescription() {
        return description;
    }
    
    /**
     * Sets the lecture's description
     * @param description The description to set.
     */
    public void setDescription(Description description) {
        this.description = description;
    }
    
    /**
     * Creates string of tasks' IDs.
     * @return String containing tasks' IDs (comma-separated)
     */
    public String toString() {
        StringBuffer sb = new StringBuffer();
        for (Iterator<Task> i = getTasks().iterator(); i.hasNext();) {
            sb.append(i.next());
            if(i.hasNext()) {
                sb.append(", ");
            }
        }
        return "tasks ["+sb+"]\n";
    }
    
    /**
     * Creates a new Task and puts it into tasks.
     * @param id ID of new task
     * @return Created task
     */
    public Task newTask(String id) throws IOException {
        Task t = new Task(this, id);
        t.getFile().createNewFile();
        tasks.put(t.getId(), t);
        return t;
    }
    
    /**
     * Returns first lecture in lectures map.
     * @return First lecture in lectures.
     */
    public Task getFirstTask() {
        if (!tasks.isEmpty()) {
            return tasks.get(tasks.firstKey());
        } else {
            return null;
        }
    }
    
    /**
     * Remove the task from tasks collection and deletes its XML file.
     * @param t task to remove
     */
    public void removeTask(Task t) {
        t.getFile().delete();
        tasks.remove(t.getId());
    }
}
