/*
 * Created on 24.2.2005
 *
 */
package net.sf.tomp.el.author.gui;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;

import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;

import net.sf.tomp.el.author.Task;
import net.sf.tomp.el.author.Course;
import net.sf.tomp.el.author.CourseTree;
import net.sf.tomp.el.author.Lecture;
import net.sf.tomp.el.author.io.AuthorIO;

import org.dom4j.DocumentException;

/**
 * @author <a href="mailto:tomp@fi.muni.cz">RNDr. Tomáš Pitner, Ph.D.</a>
 * 
 */
public class Author {

	public static final String APP_TITLE = "El Author";

	public static final String root = "/home/martin/tomp/vyuka-test/";

	public Course emptyCourse;

	public Lecture emptyLecture;

	private JFrame frame;

	private JPanel panel;

	private AuthorIO io;

	private CourseTree tree;

	private Course course;

	private Lecture lecture;
	
	private Task task;

	/**
	 * Creates a new instance of Author and calls its method main().
	 * @param args Command-line arguments
	 * @throws Exception
	 */
	public static void main(String[] args) throws Exception {
		new Author().main();
	}

	/**
	 * main() method of the whole project
	 * @throws Exception
	 */
	public void main() throws Exception {

		io = new AuthorIO();
		tree = io.loadCourseTree(root);

		Course pb138 = tree.getCourse("pb138");
		course = pb138;

		Lecture last = pb138.getLectures().newLecture("last");
		lecture = last;

		last.getDescription().setTitle("Title of the last lecture");
		last.getDescription().setBody("Body of the last lecture");
		//		io.saveLecture(last);

		CourseTree tree2 = io.loadCourseTree(root);
		Lecture last2 = tree2.getCourse("pb138").getLectures().getLecture(
				"last");
		System.out.println(tree2.getCurrentLecture());
		System.out.println(last2);

		createFrame();
		initComponents();
	}

	/**
	 * Closes the application.
	 * @author <a href="mailto:tomp@fi.muni.cz">RNDr. Tomáš Pitner, Ph.D.</a>
	 */
	protected static final class AppCloser extends WindowAdapter {
		public void windowClosing(WindowEvent e) {
			System.exit(0);
		}
	}

	private JMenuBar menuBar;
	private JMenu fileMenu, courseMenu, lectureMenu, taskMenu, helpMenu;
	
	/**
	 *  Creates the GUI menu bar.
	 *  (items File, Course, Lecture, Help)
	 *  and calls {@link #updateMenu() updateMenu} to update it.
	 */
	protected void createMenuBar() {
		// JMenuBar mb = new JMenuBar();
		// mb.add(createFileMenu());
		// mb.add(createCourseMenu());
		// lectureMenu = createLectureMenu();
		// mb.add(lectureMenu);
		// mb.add(createHelpMenu());
		// frame.setJMenuBar(mb);
		
		menuBar = new JMenuBar();
		
		frame.setJMenuBar(menuBar);
		
		fileMenu = createFileMenu();
		menuBar.add(fileMenu);
		
		courseMenu = createCourseMenu();
		menuBar.add(courseMenu);
		
		lectureMenu = createLectureMenu();
		menuBar.add(lectureMenu);

		taskMenu = createTaskMenu();
		menuBar.add(taskMenu);
		
		helpMenu = createHelpMenu();
		menuBar.add(helpMenu);
		
		updateMenu();
		
		
		frame.addWindowListener(new AppCloser());
		frame.validate();
		frame.setVisible(true);
	}

	/**
	 * Updates the menubar when i. e. a course is selected.
	 *
	 */
	protected void updateMenu() {
		
		menuBar.removeAll();
		
		fileMenu = createFileMenu();
		menuBar.add(fileMenu);
		
		courseMenu = createCourseMenu();
		menuBar.add(courseMenu);
		
		lectureMenu = createLectureMenu();
		menuBar.add(lectureMenu);
		
		taskMenu = createTaskMenu();
		menuBar.add(taskMenu);
		
		helpMenu = createHelpMenu();
		menuBar.add(helpMenu);
	}
	
	/**
	 * Dialog for selecting the directory (tree)
	 * @return Selected directory (tree) or null (directory not selected)
	 */
	protected File selectDir() {
		JFileChooser chooser = new JFileChooser();
		chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		int retval = chooser.showDialog(frame, null);
		if (retval == JFileChooser.APPROVE_OPTION) {
			return chooser.getSelectedFile();
		}
		return null;
	}

	/**
	 * Creates the GUI "File" item in menu bar
	 * (items Select Tree, Save Tree, Exit)
	 * @return Selected item
	 */
	protected JMenu createFileMenu() {
		ActionListener fml = new MenuListener();
		JMenu fileMenu = new JMenu("File");
		JMenuItem chgTree = new JMenuItem("Select Tree");
		chgTree.addActionListener(fml);
		chgTree.setActionCommand("SELECT TREE");
		fileMenu.add(chgTree);
		JMenuItem saveTree = new JMenuItem("Save Tree");
		saveTree.addActionListener(fml);
		saveTree.setActionCommand("SAVE TREE");
		fileMenu.add(saveTree);
		fileMenu.addSeparator();
		JMenuItem exit = new JMenuItem("Exit");
		exit.addActionListener(fml);
		exit.setActionCommand("EXIT");
		fileMenu.add(exit);
		return fileMenu;
	}

	/**
	 * ActionListener for "File" menu.
	 * @author <a href="mailto:tomp@fi.muni.cz">RNDr. Tomáš Pitner, Ph.D.</a>
	 *
	 */
	public class MenuListener implements ActionListener {
		/**
		 * Recognizes the performed action in menu item "File":
		 * Select - calls selectDir() to let user select a course tree
		 * Save - calls io.saveCourseTree() to save the course tree
		 * Exit - calls io.saveCourseTree() to save the course tree, 
		 * then closes the application. 
		 */
		public void actionPerformed(ActionEvent e) {
			String command = e.getActionCommand();
			if ("SELECT TREE".equals(command)) {
				try {
					File dir = selectDir();
					if (dir != null) {
						String treeId = dir.getAbsolutePath();
						changeTree(treeId);
						System.out.println(tree);
						frame.repaint();
					}
				} catch (Exception ex) {
					JOptionPane
							.showMessageDialog(frame,
									"Cannot select the tree: " + ex,
									"Cannot select the tree",
									JOptionPane.ERROR_MESSAGE);
				}
			} else if ("SAVE TREE".equals(command)) {
				try {
					io.saveCourseTree(tree);
					frame.repaint();
				} catch (Exception ex) {
					JOptionPane.showMessageDialog(frame,
							"Cannot save the tree: " + ex,
							"Cannot save the tree", JOptionPane.ERROR_MESSAGE);
				}

			} else if ("EXIT".equals(command)) {
				try {
					io.saveCourseTree(tree);
					System.exit(0);
				} catch (Exception ex) {
					JOptionPane.showMessageDialog(frame,
							"Cannot save the tree: " + ex,
							"Cannot save the tree", JOptionPane.ERROR_MESSAGE);
				}
			}
			// initComponents();
		}
	}

	/**
	 * Changes the tree. Loads the directory, course(s) and lecture(s).
	 * If the operations is successful, the message box shows the name of 
	 * the selected tree.
	 * @param id Directory path.
	 * @throws MalformedURLException
	 * @throws DocumentException
	 */
	protected void changeTree(String id) throws MalformedURLException,
			DocumentException {
		tree = io.loadCourseTree(id);
		course = tree.getCurrentCourse();
		lecture = tree.getCurrentLecture();
		frame.setTitle(APP_TITLE + " - " + tree.getDirectory());
		frame.repaint();
		JOptionPane.showMessageDialog(frame, "Selected tree "
				+ tree.getDirectory());
	}

	private JMenuItem newCourse;

	private JMenuItem saveCourse;

	private JMenuItem deleteCourse;

	/**
	 * Creates the GUI "Course" item in menu bar.
	 * (items New, Save, Delete)
	 * Items Save and Delete are shown only if a course is open.
	 * Calls addCourseMenuItems() to show the open course(s).
	 * @return Selected item
	 */
	protected JMenu createCourseMenu() {
		JMenu courseMenu = new JMenu("Course");
		newCourse = new JMenuItem("New");
		newCourse.setActionCommand("NEW COURSE");
		newCourse.addActionListener(courseListener);
		courseMenu.add(newCourse);
		if (course != null) {
			saveCourse = new JMenuItem("Save");
			saveCourse.setActionCommand("SAVE COURSE " + course.getId());
			saveCourse.addActionListener(courseListener);
			courseMenu.add(saveCourse);
			deleteCourse = new JMenuItem("Delete");
			deleteCourse.setActionCommand("DELETE COURSE " + course.getId());
			deleteCourse.addActionListener(courseListener);
			courseMenu.add(deleteCourse);
		}
		courseMenu.addSeparator();
		addCourseMenuItems(courseMenu);
		return courseMenu;
	}

	/**
	 * Updates the title and body of the course.
	 * @param c Course
	 */
	protected void updateCourse(Course c) {
		c.getDescription().setTitle(courseTitle.getText());
		c.getDescription().setBody(courseBody.getText());
	}

	private CourseListener courseListener = new CourseListener();

	/**
	 * Adds open course(s) to the "Course" menu.
	 * @param courseMenu Course menu
	 */
	protected void addCourseMenuItems(JMenu courseMenu) {
		ButtonGroup cGroup = new ButtonGroup();
		for (Course c : tree.getCourses()) {
			JRadioButtonMenuItem mi = new JRadioButtonMenuItem(c.getId() + "-"
					+ c.getDescription().getTitle());
			mi.addItemListener(courseItemListener);
			// mi.addActionListener(courseListener);
			mi.setActionCommand("SELECT COURSE " + c.getId());
			cGroup.add(mi);
			courseMenu.add(mi);
			if (c.getId() == course.getId()) {
				mi.setSelected(true);
			}
			mi = null;
		}
	}
	
	/**
	 * 
	 * ItemListener for "Course" menu RadioButtonMenuItems.
	 * @author <a href="mailto:xkyselak@fi.muni.cz">Martin Kyselák</a>
	 *
	 */
	public class CourseItemListener implements ItemListener {
		public void itemStateChanged(ItemEvent e) {
			if (e.getStateChange() == 1) {
				String command = ((JRadioButtonMenuItem) e.getItem()).getText();
				//if (command.startsWith("SELECT LECTURE ")) {
				selectCourse(command.substring(0,command.indexOf('-')));
				// System.out.println(((JRadioButtonMenuItem) e.getItem()).getText());
				// selectLecture(command.substring("SELECT LECTURE ".length()));
				//}
			}
		}
	}
	
	private CourseItemListener courseItemListener = new CourseItemListener();
	
	/**
	 * ActionListener for "Course" menu.<br />
	 * - this listener is for JMenuItems only<br />
	 * - for RadioButtonMenuItems is implemented 
	 * {@link net.sf.tomp.el.author.gui.Author.CourseItemListener 
	 * CourseItemListener}
	 * @author <a href="mailto:tomp@fi.muni.cz">RNDr. Tomáš Pitner, Ph.D.</a>
	 * @author <a href="mailto:xkyselak@fi.muni.cz">Martin Kyselák</a>
	 */
	public class CourseListener implements ActionListener {
		
		/**
		 * Recognizes the performed action in menu item "Course":
		 * Select - calls selectCourse() to select a course
		 * Save - calls io.saveCourse() to save the course
		 * New - calls tree.newCourse() to create a new course
	     * Delete - ?
		 */
		public void actionPerformed(ActionEvent e) {
			String command = e.getActionCommand();

			// if (command.startsWith("SELECT COURSE ")) {
			//	selectCourse(command.substring("SELECT COURSE ".length()));
            //
			// } else 
			if (command.startsWith("SAVE COURSE ")) {
				Course c = tree.getCourse(command.substring("SAVE COURSE "
						.length()));
				updateCourse(c);
				try {
					io.saveCourse(c);
					JOptionPane.showMessageDialog(frame, "Course "
							+ c.getDescription().getTitle() + " saved");
				} catch (Exception ex) {
					JOptionPane.showMessageDialog(frame,
							"Cannot save the course " + c + ex,
							"Cannot save the course " + c + ex,
							JOptionPane.ERROR_MESSAGE);
				}

			} else if (command.equals("NEW COURSE")) {
				String newId = JOptionPane.showInputDialog("New course id: ");
				if (newId != null && newId.length() > 0
						&& tree.getCourse(newId) == null) {
					try {
						Course c = tree.newCourse(newId);
						io.saveCourse(c);
						selectCourse(c.getId());
						course = c;
					} catch(IOException ex) {
						JOptionPane.showMessageDialog(frame, 
								"Can't create main-info.xml file for course");
					}  catch (Exception ex) {
						JOptionPane.showMessageDialog(frame,
								"Cannot save the course " + ex,
								"Cannot save the course " + ex,
								JOptionPane.ERROR_MESSAGE);
					}
					
				}

			} else if (command.startsWith("DELETE COURSE ")) {
				String deletedId = command.substring("DELETE COURSE ".length());
				if (JOptionPane.YES_OPTION == JOptionPane.showConfirmDialog(
						frame, "Really delete course "
								+ tree.getCourse(deletedId).getDescription()
										.getTitle() + "?", "Delete course?",
						JOptionPane.YES_NO_OPTION)) {
					JOptionPane.showMessageDialog(frame, "Course "
							+ tree.getCourse(deletedId).getDescription()
									.getTitle() + " would be deleted");
					tree.removeCourse(course);
					if (!tree.getCourses().isEmpty()) {
						selectCourse(tree.getFirstCourse().getId());
					}
				}
			}
		}
	}

	/**
	 * Selects the course. If the operations is successful, 
	 * the message box shows the name of the selected course.
	 * @param id Course ID
	 */
	protected void selectCourse(String id) {
		Course selectedCourse = tree.getCourse(id);
		if (selectedCourse == null) {
			JOptionPane.showMessageDialog(frame, "No course selected");
		} else {
			course = selectedCourse;
			
			// update coursemenu
			if (menuBar.getComponentCount() > 1) {
				menuBar.remove(1);
				menuBar.add(createCourseMenu(),1);
			}
			
			// update lecturemenu and tab
			if (menuBar.getComponentCount() > 2) {
				menuBar.remove(2);
				menuBar.add(createLectureMenu(),2);
			}
			
			if (course.getLectures().getFirstLecture() != null) {
				selectLecture(course.getLectures().getFirstLecture().getId());
			}
			
			// update taskmenu and tab
			if (menuBar.getComponentCount() > 3) {
				menuBar.remove(3);
				menuBar.add(createTaskMenu(),3);
			}
			
			if (course.getTasks().getFirstTask() != null) {
				selectTask(course.getTasks().getFirstTask().getId());
			}
			
			updatePanel();
			tabbedpane.setSelectedIndex(1);
			
			// update helpmenu
			if (menuBar.getComponentCount() > 4) {
				menuBar.remove(4);
				menuBar.add(createHelpMenu(),4);
			}
			
			
			
			frame.repaint();
			// JOptionPane.showMessageDialog(frame, "Selected course "
			//		+ selectedCourse.getId());
			
		}
	}

	private JMenuItem newLecture, saveLecture, deleteLecture;

	/**
	 * Creates the GUI "Lecture" item in menu bar.
	 * (items New, Save, Delete)
	 * Items Save and Delete are shown only if a course is open.
	 * Calls {@link net.sf.tomp.el.author.gui.Author#addLectureMenuItems
	 * addLectureMenuItems} to show the lectures of the selected course.
	 * @return Selected item
	 */
	protected JMenu createLectureMenu() {
		JMenu lectureMenu = new JMenu("Lecture");
		newLecture = new JMenuItem("New");
		lectureMenu.add(newLecture);
		newLecture.setActionCommand("NEW LECTURE");
		newLecture.addActionListener(lectureListener);
		if (lecture != null) {
			saveLecture = new JMenuItem("Save");
			saveLecture.setActionCommand("SAVE LECTURE " + lecture.getId());
			saveLecture.addActionListener(lectureListener);
			lectureMenu.add(saveLecture);
			deleteLecture = new JMenuItem("Delete");
			deleteLecture.setActionCommand("DELETE LECTURE " + lecture.getId());
			deleteLecture.addActionListener(lectureListener);
			lectureMenu.add(deleteLecture);
		}
		lectureMenu.addSeparator();
		addLectureMenuItems(lectureMenu);
		return lectureMenu;
	}

	
	/**
	 * Adds lectures of selected course to the "Lecture" menu.
	 * @param lecturesMenu Lectures menu
	 */
	protected void addLectureMenuItems(JMenu lecturesMenu) {
		if (course != null) {

			ButtonGroup lGroup = new ButtonGroup();
			for (Lecture c : course.getLectures().getLectures()) {
				JRadioButtonMenuItem mi = new JRadioButtonMenuItem(c.getId() 
						+ "-" + c.getDescription().getTitle());
				// mi.setActionCommand("SELECT LECTURE " + c.getId());
				mi.addItemListener(lectureItemListener);
				// mi.addActionListener(lectureListener);
				lGroup.add(mi);
				lecturesMenu.add(mi);
				if (c.getId() == lecture.getId()) {
					mi.setSelected(true);
				}
				mi = null;
			}
		}
	}

	/**
	 * Updates the title and body of the lecture.
	 * @param lect Lecture
	 */
	protected void updateLecture(Lecture lect) {
		lect.getDescription().setTitle(lectureTitle.getText());
		lect.getDescription().setBody(lectureBody.getText());
	}

	private LectureItemListener lectureItemListener = new LectureItemListener();
	
	/**
	 * 
	 * ItemListener for "Lecture" menu RadioButtonMenuItems.
	 * @author <a href="mailto:xkyselak@fi.muni.cz">Martin Kyselák</a>
	 *
	 */
	public class LectureItemListener implements ItemListener {
		public void itemStateChanged(ItemEvent e) {
			if (e.getStateChange() == 1) {
				String command = ((JRadioButtonMenuItem) e.getItem()).getText();
				
				//	if (command.startsWith("SELECT LECTURE ")) {
				System.out.println(command.substring(0,command.indexOf('-')));
				selectLecture(command.substring(0,command.indexOf('-')));
				// selectLecture(command.substring("SELECT LECTURE ".length()));
					//}
			}
		}
	}
	
	private LectureListener lectureListener = new LectureListener();

	/**
	 * ActionListener for "Lecture" menu.<br />
	 * - this listener is for JMenuItems only<br />
	 * - for RadioButtonMenuItems is implemented 
	 * {@link net.sf.tomp.el.author.gui.Author.LectureItemListener 
	 * LectureItemListener}
	 * @author <a href="mailto:tomp@fi.muni.cz">RNDr. Tomáš Pitner, Ph.D.</a>
	 * @author <a href="mailto:xkyselak@fi.muni.cz">Martin Kyselák</a>
	 */
	public class LectureListener implements ActionListener {
		/**
		 * Recognizes the performed action in menu item "Lecture":<br />
		 * Select - calls {@link net.sf.tomp.el.author.gui.Author#selectLecture 
		 * selectLecture} to select a lecture<br />
		 * Save - calls {@link net.sf.tomp.el.author.io.AuthorIO#saveLecture 
		 * saveLecture} to save the lecture<br />
		 * New - calls {@link net.sf.tomp.el.author.Lectures#newLecture newLecture}
		 * to create a new lecture<br />
	     * Delete - calls {@link net.sf.tomp.el.author.Lectures#removeLecture 
	     * removeLecture} to delete a lecture
		 */
		public void actionPerformed(ActionEvent e) {
			String command = e.getActionCommand();
			if (command.startsWith("SAVE LECTURE ")) {
				Lecture lect = course.getLectures().getLecture(
						command.substring("SAVE LECTURE ".length()));
				updateLecture(lect);
				try {
					updateLecture(lect);
					if (lect != null) {
						io.saveLecture(lect);
						JOptionPane.showMessageDialog(frame, "Lecture "
								+ lect.getDescription().getTitle()
								+ " was saved");
					}
				} catch (Exception ex) {
					JOptionPane.showMessageDialog(frame,
							"Cannot save the lecture " + lect + ex,
							"Cannot save the lecture " + lect + ex,
							JOptionPane.ERROR_MESSAGE);
				}

			} else if (command.equals("NEW LECTURE")) {
				String newId = JOptionPane.showInputDialog("New lecture id: ");
				if (newId != null && newId.length() > 0
						&& course.getLectures().getLecture(newId) == null) {
					try {
						Lecture lect = course.getLectures().newLecture(newId);
						io.saveLecture(lect);
						selectLecture(lect.getId());
						lecture = lect;
					} catch(IOException ex) {
						JOptionPane.showMessageDialog(frame, 
							"Can't create XML file for new lecture.");
					}  catch (Exception ex) {
						JOptionPane.showMessageDialog(frame,
								"Cannot save the lecture " + ex,
								"Cannot save the lecture " + ex,
								JOptionPane.ERROR_MESSAGE);
					}
				}

			} else if (command.startsWith("DELETE LECTURE ")) {
				try {
					Lecture lect = course.getLectures().getLecture(
							command.substring("DELETE LECTURE ".length()));
					if (lect != null) {
						if (JOptionPane.YES_OPTION == JOptionPane
								.showConfirmDialog(frame,
										"Really delete lecture "
												+ lect.getDescription()
														.getTitle() + "?",
										"Delete lecture?",
										JOptionPane.YES_NO_OPTION)) {
							JOptionPane.showMessageDialog(frame, "Lecture "
									+ lect.getDescription().getTitle()
									+ " would be deleted");
							course.getLectures().removeLecture(
								course.getLectures().getLecture(
									lecture.getId()
									)
								);
							if (!tree.getCourses().isEmpty()) {
								selectCourse(tree.getFirstCourse().getId());
							}
						}

					}
				} catch (Exception ex) {

				}

			} else {

			}
			// initComponents();
		}
	}

	/**
	 * Selects the lecture. If the operations is successful, 
	 * the message box shows the name of the selected lecture. 
	 * @param id ID of lecture to select
	 */
	protected void selectLecture(String id) {
		Lecture selectedLecture = course.getLectures().getLecture(id);
		if (selectedLecture == null) {
			JOptionPane.showMessageDialog(frame, "No lecture selected");
		} else {
			lecture = selectedLecture;
			
			// update lecturemenu and tab
			if (menuBar.getComponentCount() > 2) {
				menuBar.remove(2);
				menuBar.add(createLectureMenu(),2);
			}

			updatePanel();
			tabbedpane.setSelectedIndex(2);

			// update taskmenu
			if (menuBar.getComponentCount() > 3) {
				menuBar.remove(3);
				menuBar.add(createTaskMenu(),3);
			}
			
			// update helpmenu
			if (menuBar.getComponentCount() > 4) {
				menuBar.remove(4);
				menuBar.add(createHelpMenu(),4);
			}
			
			// JOptionPane.showMessageDialog(frame, "Selected lecture "
			//		+ selectedLecture.getId());
		}
	}

	private JMenuItem newTask, saveTask, deleteTask;

	/**
	 * Creates the GUI "Task" item in menu bar.
	 * (items New, Save, Delete)
	 * Items Save and Delete are shown only if a course is open.
	 * Calls {@link net.sf.tomp.el.author.gui.Author#addTaskMenuItems
	 * addTaskMenuItems} to show the tasks of the selected course.
	 * @return Selected item
	 */
	protected JMenu createTaskMenu() {
		JMenu taskMenu = new JMenu("Task");
		newTask = new JMenuItem("New");
		taskMenu.add(newTask);
		newTask.setActionCommand("NEW TASK");
		newTask.addActionListener(taskListener);
		if (task != null) {
			saveTask = new JMenuItem("Save");
			saveTask.setActionCommand("SAVE TASK " + task.getId());
			saveTask.addActionListener(taskListener);
			taskMenu.add(saveTask);
			deleteTask = new JMenuItem("Delete");
			deleteTask.setActionCommand("DELETE TASK " + task.getId());
			deleteTask.addActionListener(taskListener);
			taskMenu.add(deleteTask);
		}
		taskMenu.addSeparator();
		addTaskMenuItems(taskMenu);
		return taskMenu;
	}

	
	/**
	 * Adds tasks of selected course to the "Task" menu.
	 * @param tasksMenu Tasks menu
	 */
	protected void addTaskMenuItems(JMenu tasksMenu) {
		if (course != null) {

			ButtonGroup lGroup = new ButtonGroup();
			for (Task t : course.getTasks().getTasks()) {
				JRadioButtonMenuItem mi = new JRadioButtonMenuItem(t.getId() 
						+ "-" + t.getDescription().getTitle());
				// mi.setActionCommand("SELECT LECTURE " + c.getId());
				mi.addItemListener(taskItemListener);
				// mi.addActionListener(taskListener);
				lGroup.add(mi);
				tasksMenu.add(mi);
				if (t.getId() == task.getId()) {
					mi.setSelected(true);
				}
				mi = null;
			}
		}
	}

	/**
	 * Updates the title and body of the task.
	 * @param t Task
	 */
	protected void updateTask(Task t) {
		t.getDescription().setTitle(taskTitle.getText());
		t.getDescription().setBody(taskBody.getText());
	}

	private TaskItemListener taskItemListener = new TaskItemListener();
	
	/**
	 * 
	 * ItemListener for "Task" menu RadioButtonMenuItems.
	 * @author <a href="mailto:xkyselak@fi.muni.cz">Martin Kyselák</a>
	 *
	 */
	public class TaskItemListener implements ItemListener {
		public void itemStateChanged(ItemEvent e) {
			if (e.getStateChange() == 1) {
				String command = ((JRadioButtonMenuItem) e.getItem()).getText();
				
				//	if (command.startsWith("SELECT LECTURE ")) {
				System.out.println(command.substring(0,command.indexOf('-')));
				selectTask(command.substring(0,command.indexOf('-')));
				// selectTask(command.substring("SELECT LECTURE ".length()));
					//}
			}
		}
	}
	
	private TaskListener taskListener = new TaskListener();

	/**
	 * ActionListener for "Task" menu.<br />
	 * - this listener is for JMenuItems only<br />
	 * - for RadioButtonMenuItems is implemented 
	 * {@link net.sf.tomp.el.author.gui.Author.TaskItemListener 
	 * TaskItemListener}
	 * @author <a href="mailto:tomp@fi.muni.cz">RNDr. Tomáš Pitner, Ph.D.</a>
	 * @author <a href="mailto:xkyselak@fi.muni.cz">Martin Kyselák</a>
	 */
	public class TaskListener implements ActionListener {
		/**
		 * Recognizes the performed action in menu item "Task":<br />
		 * Select - calls {@link net.sf.tomp.el.author.gui.Author#selectTask 
		 * selectTask} to select a task<br />
		 * Save - calls {@link net.sf.tomp.el.author.io.AuthorIO#saveTask 
		 * saveTask} to save the task<br />
		 * New - calls {@link net.sf.tomp.el.author.Tasks#newTask newTask}
		 * to create a new task<br />
	     * Delete - calls {@link net.sf.tomp.el.author.Tasks#removeTask 
	     * removeTask} to delete a task
		 */
		public void actionPerformed(ActionEvent e) {
			String command = e.getActionCommand();
			if (command.startsWith("SAVE TASK ")) {
				Task t = course.getTasks().getTask(
						command.substring("SAVE TASK ".length()));
				updateTask(t);
				try {
					updateTask(t);
					if (t != null) {
						io.saveTask(t);
						JOptionPane.showMessageDialog(frame, "Task "
								+ t.getDescription().getTitle()
								+ " was saved");
					}
				} catch (Exception ex) {
					JOptionPane.showMessageDialog(frame,
							"Cannot save the task " + t + ex,
							"Cannot save the task " + t + ex,
							JOptionPane.ERROR_MESSAGE);
				}

			} else if (command.equals("NEW TASK")) {
				String newId = JOptionPane.showInputDialog("New task id: ");
				if (newId != null && newId.length() > 0
						&& course.getTasks().getTask(newId) == null) {
					try {
						Task t = course.getTasks().newTask(newId);
						io.saveTask(t);
						selectTask(t.getId());
						task = t;
					} catch(IOException ex) {
						JOptionPane.showMessageDialog(frame, 
							"Can't create XML file for new task.");
					}  catch (Exception ex) {
						JOptionPane.showMessageDialog(frame,
								"Cannot save the task " + ex,
								"Cannot save the task " + ex,
								JOptionPane.ERROR_MESSAGE);
					}
				}

			} else if (command.startsWith("DELETE TASK ")) {
				try {
					Task t = course.getTasks().getTask(
							command.substring("DELETE TASK ".length()));
					if (t != null) {
						if (JOptionPane.YES_OPTION == JOptionPane
								.showConfirmDialog(frame,
										"Really delete task "
												+ t.getDescription()
														.getTitle() + "?",
										"Delete task?",
										JOptionPane.YES_NO_OPTION)) {
							JOptionPane.showMessageDialog(frame, "Task "
									+ t.getDescription().getTitle()
									+ " would be deleted");
							course.getTasks().removeTask(
								course.getTasks().getTask(
									t.getId()
									)
								);
							if (!tree.getCourses().isEmpty()) {
								selectCourse(tree.getFirstCourse().getId());
							}
						}

					}
				} catch (Exception ex) {

				}

			} else {

			}
			// initComponents();
		}
	}

	/**
	 * Selects the task. If the operations is successful, 
	 * the message box shows the name of the selected task. 
	 * @param id ID of task to select
	 */
	protected void selectTask(String id) {
		Task selectedTask = course.getTasks().getTask(id);
		if (selectedTask == null) {
			JOptionPane.showMessageDialog(frame, "No task selected");
		} else {
			task = selectedTask;
			
			updatePanel();
			tabbedpane.setSelectedIndex(3);

			// update taskmenu
			if (menuBar.getComponentCount() > 3) {
				menuBar.remove(4);
				menuBar.add(createTaskMenu(),3);
			}
			
			// update helpmenu
			if (menuBar.getComponentCount() > 4) {
				menuBar.remove(4);
				menuBar.add(createHelpMenu(),4);
			}
			
			// JOptionPane.showMessageDialog(frame, "Selected task "
			//		+ selectedTask.getId());
		}
	}
	
	/**
	 * Creates the GUI "Help" item in menu bar.
	 * (item Content)
	 * @return Selected item
	 */
	protected JMenu createHelpMenu() {
		JMenu helpMenu = new JMenu("Help");
		JMenuItem content = new JMenuItem("Content");
		
		// POKUS MENU
		/* ButtonGroup lGroup = new ButtonGroup();
		JRadioButtonMenuItem mi = new JRadioButtonMenuItem("jedna");
		lGroup.add(mi);
		helpMenu.add(mi);
		JRadioButtonMenuItem mi2 = new JRadioButtonMenuItem("dva");
		lGroup.add(mi2);
		helpMenu.add(mi2);
		JRadioButtonMenuItem mi3 = new JRadioButtonMenuItem("tri");
		lGroup.add(mi3);
		helpMenu.add(mi3); */
		
		helpMenu.add(content);
		return helpMenu;
	}

	private JTabbedPane tabbedpane;

	private JPanel treePanel, coursePanel, lecturePanel, practicesPanel,
			taskPanel, projectPanel;

	/**
	 * Creates the GUI "Tree" panel.
	 * (Root directory, Current term, Current course, Current lecture) 
	 * @return created Panel
	 */
	protected JPanel createTreePanel() {
		String name = "Tree";
		JLabel pix = new JLabel(name);
		treePanel = new JPanel();
		treePanel.setLayout(new GridLayout(5, 2));
		JLabel treeDir = new JLabel("Root directory: ");
		JTextField df = new JTextField(tree.getDirectory().getAbsolutePath());
		treePanel.add(treeDir);
		treePanel.add(df);
		JLabel ct = new JLabel("Current term: ");
		JTextField ctf = new JTextField(tree.getCurrentTerm());
		treePanel.add(ct);
		treePanel.add(ctf);
		JLabel cc = new JLabel("Current course: ");
		Course cCourse = tree.getCurrentCourse();
		JTextField ccf = new JTextField(cCourse == null ? "" : cCourse
				.getDescription().getTitle());
		treePanel.add(cc);
		treePanel.add(ccf);
		JLabel cl = new JLabel("Current lecture: ");
		Lecture cLecture = tree.getCurrentLecture();
		JTextField clf = new JTextField(cLecture == null ? "" : cLecture
				.getDescription().getTitle());
		treePanel.add(cl);
		treePanel.add(clf);
		JButton saveB = new JButton("Save");
		JButton cancelB = new JButton("Cancel");
		treePanel.add(saveB);
		treePanel.add(cancelB);
		tabbedpane.add(name, treePanel);
		
		return treePanel;
	}

	private JTextField courseTitle, courseBody;
	
	/**
	 * Creates the GUI "Course" panel.
	 * (Course id, Title, Description) 
	 * @return created panel
	 */
	protected JPanel createCoursePanel() {
		// String name = course == null ? "Course" : "Course " + course.getId();
		// coursePanelLabel = new JLabel();
		// coursePanelLabel.setText(name);
		// JLabel pix = new JLabel(name);
		coursePanel = new JPanel();
		coursePanel.setLayout(new GridLayout(4, 2));
		JLabel idl = new JLabel("Course id: ");
		JTextField idf = new JTextField(course == null ? "none" : course
				.getId());
		coursePanel.add(idl);
		coursePanel.add(idf);
		JLabel ct = new JLabel("Title: ");
		courseTitle = new JTextField(course == null ? "none" : course
				.getDescription().getTitle());
		coursePanel.add(ct);
		coursePanel.add(courseTitle);
		JLabel cd = new JLabel("Description: ");
		courseBody = new JTextField(course == null ? "none" : course
				.getDescription().getBody());
		coursePanel.add(cd);
		coursePanel.add(courseBody);
		if (course != null) {
			JButton saveB = new JButton("Save");
			saveB.setActionCommand("SAVE COURSE " + course.getId());
			saveB.addActionListener(courseListener);
			JButton cancelB = new JButton("Reload");
			cancelB.setActionCommand("RELOAD COURSE " + course.getId());
			cancelB.addActionListener(courseListener);
			coursePanel.add(saveB);
			coursePanel.add(cancelB);
		}
		// tabbedpane.add(name, coursePanel);
		
		return coursePanel;
	}

	private JTextField lectureTitle, lectureBody;

	/**
	 * Creates the GUI "Lecture" panel.
	 * (Lecture id, Title, Description)
	 * @return created panel 
	 */
	protected JPanel createLecturePanel() {
		String name = lecture == null ? "Lecture" : "Lecture "
				+ lecture.getId();
		JLabel pix = new JLabel(name);
		lecturePanel = new JPanel();
		lecturePanel.setLayout(new GridLayout(4, 2));
		JLabel idl = new JLabel("Lecture id: ");
		JTextField idf = new JTextField(lecture == null ? "none" : lecture
				.getId());
		lecturePanel.add(idl);
		lecturePanel.add(idf);
		JLabel ct = new JLabel("Title: ");
		lectureTitle = new JTextField(lecture == null ? "none" : lecture
				.getDescription().getTitle());
		lecturePanel.add(ct);
		lecturePanel.add(lectureTitle);
		JLabel cd = new JLabel("Description: ");
		lectureBody = new JTextField(lecture == null ? "none" : lecture
				.getDescription().getBody());
		lecturePanel.add(cd);
		lecturePanel.add(lectureBody);
		if (lecture != null) {
			JButton saveB = new JButton("Save");
			saveB.setActionCommand("SAVE LECTURE " + lecture.getId());
			saveB.addActionListener(lectureListener);
			JButton cancelB = new JButton("Reload");
			cancelB.setActionCommand("RELOAD LECTURE " + lecture.getId());
			cancelB.addActionListener(lectureListener);
			lecturePanel.add(saveB);
			lecturePanel.add(cancelB);
		}
		tabbedpane.add(name, lecturePanel);
		
		return lecturePanel;
	}

	/**
	 * Creates the GUI "Practises" panel.
	 * (no items yet) 
	 * @return created tab
	 */
	protected JPanel createPracticesPanel() {
		String name = "Practices";
		JLabel pix = new JLabel(name);
		practicesPanel = new JPanel();
		tabbedpane.add(name, practicesPanel);
		return practicesPanel;
	}

	private JTextField taskTitle, taskBody;
	
	/**
	 * Creates the GUI "Task" panel.
	 * (no items yet) 
	 * @return created tab
	 */
	protected JPanel createTaskPanel() {
		String name = task == null ? "Task" : "Task "
			+ task.getId();
		JLabel pix = new JLabel(name);
		taskPanel = new JPanel();
		taskPanel.setLayout(new GridLayout(4, 2));
		JLabel idl = new JLabel("Task id: ");
		JTextField idf = new JTextField(task == null ? "none" : task
				.getId());
		taskPanel.add(idl);
		taskPanel.add(idf);
		JLabel ct = new JLabel("Title: ");
		taskTitle = new JTextField(task == null ? "none" : task
				.getDescription().getTitle());
		taskPanel.add(ct);
		taskPanel.add(taskTitle);
		JLabel cd = new JLabel("Description: ");
		taskBody = new JTextField(task == null ? "none" : task
				.getDescription().getBody());
		taskPanel.add(cd);
		taskPanel.add(taskBody);
		if (task != null) {
			JButton saveB = new JButton("Save");
			saveB.setActionCommand("SAVE TASK " + task.getId());
			saveB.addActionListener(taskListener);
			JButton cancelB = new JButton("Reload");
			cancelB.setActionCommand("RELOAD TASK " + task.getId());
			cancelB.addActionListener(taskListener);
			taskPanel.add(saveB);
			taskPanel.add(cancelB);
		}
		tabbedpane.add(name, taskPanel);
		
		return taskPanel;
	}
	
	/**
	 * Creates the GUI "Project" panel.
	 * (no items yet) 
	 * @return created tab
	 */
	protected JPanel createProjectPanel() {
		String name = "Project";
		JLabel pix = new JLabel(name);
		projectPanel = new JPanel();
		tabbedpane.add(name, projectPanel);

		return projectPanel;
	}

	/**
	 * Updates the tabbedpane when i. e. a course is selected.
	 *
	 */
	protected void updatePanel() {
		// updating tabbedpane
		tabbedpane.removeAll();
		
		// updating tree tab 			
		tabbedpane.add(createTreePanel(),0);
		tabbedpane.setTitleAt(0,"Tree");
		
		// updating course tab
		tabbedpane.add(createCoursePanel(),1);
		if (course != null) {
			tabbedpane.setTitleAt(1,"Course: " + course.getId());
		} else {
			tabbedpane.setTitleAt(1,"Course: none");
		}
		
		// updating lecture tab
		tabbedpane.add(createLecturePanel(),2);
		if (lecture != null) {
			tabbedpane.setTitleAt(2,"Lecture: " + lecture.getId());
		} else {
			tabbedpane.setTitleAt(2,"Lecture: none");
		}
		
		// updating practises tab
		tabbedpane.add(createPracticesPanel(),3);
		tabbedpane.setTitleAt(3,"Practices");
		
		// updating task tab
		tabbedpane.add(createTaskPanel(),4);
		tabbedpane.setTitleAt(4,"Task");
		
		// updating project tab
		tabbedpane.add(createProjectPanel(),5);
		tabbedpane.setTitleAt(5,"Project");
		
		// selecting first tab
		tabbedpane.setSelectedIndex(0);
		
		frame.repaint();
	}
	
	/**
	 *  Creates the GUI panel
	 *  (panels Tree, Course, Lecture, Practices, Task, Project)
	 *  and calls {@link #updatePanel() updatePanel} to update it.
	 *  
	 */
	protected void createPanel() {
		panel = new JPanel();
		panel.setLayout(new BorderLayout());
		
		// creating tabbedpane
		tabbedpane = new JTabbedPane();
		panel.add(tabbedpane, BorderLayout.CENTER);

		// updating tabbedpane
		updatePanel();

		frame.getContentPane().setLayout(new BorderLayout());
		frame.getContentPane().add(panel, BorderLayout.CENTER);
	}

	/**
	 * Creates the application's main GUI frame.
	 */
	protected void createFrame() {
		frame = new JFrame("El Author - " + root);

		frame.setSize(600, 300);
		frame.setLocation(100, 100);
	}
	
	/**
	 * Initializes the GUI menu and panel.
	 */
	protected void initComponents() {
		createPanel();
		createMenuBar();
	}
}
