/*
 * Created on 9.5.2005
 *
 */
package net.sf.tomp.el.is;

import java.io.File;
import java.io.FilenameFilter;

/**
 * @author tomp
 *
 */
public class FileUtils {
    
    protected static final String INFO_EXT = "-info.xml";

    protected static final String PARAM_PREF = "params-";

    public static File[] listFilenameEndsWith(File dir, final String endsWith) {
        return dir.listFiles(new FilenameFilter() {
            public boolean accept(File dir, String name) {
                return name.endsWith(endsWith);
            }
        });
    }

    public static File[] listFilenameStartsWith(File dir,
            final String startsWith) {
        return dir.listFiles(new FilenameFilter() {
            public boolean accept(File dir, String name) {
                return name.startsWith(startsWith);
            }
        });
    }

    public static File[] listFilenameStartsWithEndsWith(File dir,
            final String startsWith, final String endsWith) {
        return dir.listFiles(new FilenameFilter() {
            public boolean accept(File dir, String name) {
                return name.startsWith(startsWith) && name.endsWith(endsWith);
            }
        });
    }

    public static File[] listPrincipalFiles(File dir) {
        return listFilenameEndsWith(dir, INFO_EXT);
    }

    public static String[] listPrincipalNames(File dir) {
        File[] ff = listPrincipalFiles(dir);
        String[] ss = new String[ff.length];
        for(int i = 0; i < ss.length; i++) {
        	String fn = ff[i].getName();
            ss[i] = fn.substring(0, fn.length() - INFO_EXT.length());
        }
        return ss;
    }

    public static File[] listParameterDirFiles(File dir) {
        return listFilenameStartsWith(dir, PARAM_PREF);
    }

    public static String[] listParameterDirNames(File dir) {
        File[] ff = listParameterDirFiles(dir);
        String[] ss = new String[ff.length];
        for(int i = 0; i < ss.length; i++) { 
        	String fn = ff[i].getName();
            ss[i] = fn.substring(PARAM_PREF.length());
        }
        return ss;
    }

}
